@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1999, 2013 Tanuki Software, Ltd.
+ * Copyright (c) 1999, 2014 Tanuki Software, Ltd.
  * http://www.tanukisoftware.com
  * All rights reserved.
  *
@@ -197,6 +197,21 @@ typedef unsigned long TICKS;
 #define CTRL_CODE_QUEUE_SIZE 26 /* Can enqueue one less than this count at any time. */
 #endif
 
+#define WRAPPER_JAVAIO_BUFFER_SIZE_SYSTEM_DEFAULT 0
+#define WRAPPER_JAVAIO_BUFFER_SIZE_MIN 1024
+#define WRAPPER_JAVAIO_BUFFER_SIZE_MAX (10 * 1024 * 1024)
+#define WRAPPER_JAVAIO_BUFFER_SIZE_DEFAULT (64 * 1024)
+
+/*#define DEBUG_PING_QUEUE*/
+#define WRAPPER_MAX_PENDING_PINGS 10
+typedef struct PendingPing PendingPing, *PPendingPing;
+struct PendingPing {
+    TICKS sentTicks;
+    TICKS timeoutTicks;
+    TICKS slowTicks;
+    PPendingPing nextPendingPing;
+};
+
 /* Type definitions */
 typedef struct WrapperConfig WrapperConfig;
 struct WrapperConfig {
@@ -217,6 +232,7 @@ struct WrapperConfig {
     int     backendType;            /* The type of the backend that the Wrapper and Java use to communicate. */
     int     configured;             /* TRUE if loadConfiguration has been called. */
     int     useSystemTime;          /* TRUE if the wrapper should use the system clock for timing, FALSE if a tick counter should be used. */
+    int     logBufferGrowth;        /* TRUE if changes to internal buffer sizes should be logged. */
     int     timerFastThreshold;     /* If the difference between the system time based tick count and the timer tick count ever falls by more than this value then a warning will be displayed. */
     int     timerSlowThreshold;     /* If the difference between the system time based tick count and the timer tick count ever grows by more than this value then a warning will be displayed. */
     int     useTickMutex;           /* TRUE if access to the tick count should be protected by a mutex. */
@@ -238,8 +254,10 @@ struct WrapperConfig {
     int     commandLogLevel;        /* The log level to use when logging the java command. */
     int     printJVMVersion;        /* tells the Wrapper to create a temp JVM to query the version, before starting the java application */
 #ifdef WIN32
+    TCHAR   *jvmVersionCommand;     /* Command used to launch the JVM and request its version */
     TCHAR   *jvmCommand;            /* Command used to launch the JVM */
 #else /* UNIX */
+    TCHAR   **jvmVersionCommand;    /* Command used to launch the JVM and request its version */
     TCHAR   **jvmCommand;           /* Command used to launch the JVM */
 #endif
     int     detachStarted;          /* TRUE if the JVM process should be detached once it has started. */
@@ -261,7 +279,10 @@ struct WrapperConfig {
     int     shutdownTimeout;        /* Number of seconds the wrapper will wait for a JVM to shutdown */
     int     jvmExitTimeout;         /* Number of seconds the wrapper will wait for a JVM to process to terminate */
     int     jvmCleanupTimeout;      /* Number of seconds the wrapper will allow for its post JVM shudown cleanup. */
-    int     jvmTerminateTimeout;      /* Number of seconds the wrapper will allow for the JVM to respond to TerminateProcess request. */
+    int     jvmTerminateTimeout;    /* Number of seconds the wrapper will allow for the JVM to respond to TerminateProcess request. */
+#ifdef WIN32
+    int     javaIOBufferSize;       /* Size of the pipe buffer to use for java I/O. */
+#endif
     int     useJavaIOThread;        /* If TRUE then a dedicated thread will be used to process console output form the JVM. */
     int     pauseThreadMain;        /* Number of seconds to pause the main thread on its next loop.  Only used for testing. */
     int     pauseThreadTimer;       /* Number of seconds to pause the timer thread on its next loop.  Only used for testing. */
@@ -304,8 +325,12 @@ struct WrapperConfig {
     int     isPageFaultOutputEnabled;/* TRUE if detailed PageFault output should be included in status output. */
     int     pageFaultOutputInterval;/* Interval in seconds at which PageFault usage is logged. */
     TICKS   pageFaultOutputTimeoutTicks; /* Tick count at which PageFault will next be logged. */
-    int     logfileInactivityTimeout; /* The number of seconds of inactivity before the logfile will be closed. */
-    TICKS   logfileInactivityTimeoutTicks; /* Tick count at which the logfile will be considered inactive and closed. */
+    int     logfileFlushTimeout;    /* The number of seconds before the logfile will be flushed. */
+    TICKS   logfileFlushTimeoutTicks; /* Tick count at which the logfile will be flushed. */
+    int     logfileFlushTimeoutTicksSet; /* TRUE if logfileFlushTimeoutTicks is set. */
+    int     logfileCloseTimeout;    /* The number of seconds of inactivity before the logfile will be closed. */
+    TICKS   logfileCloseTimeoutTicks; /* Tick count at which the logfile will be considered inactive and closed. */
+    int     logfileCloseTimeoutTicksSet; /* TRUE if logfileCloseTimeoutTicks is set. */
     int     isTestsDisabled;        /* TRUE if the use of tests in the WrapperManager class should be disabled. */
     int     isShutdownHookDisabled; /* TRUE if the use of a shutdown hook by the WrapperManager class should be disabled. */
     int     isForcedShutdownDisabled; /* TRUE if forced shutdowns are disabled. */
@@ -334,6 +359,7 @@ struct WrapperConfig {
     int     *outputFilterAllowWildFlags; /* Array of output filter flags that say whether or not wild cards in the filter can be processed. */
     size_t  *outputFilterMinLens;   /* Array of the minimum text lengths that could possibly match the specified filter.  Only used if it contains wildcards. */
     TCHAR   *pidFilename;           /* Name of file to store wrapper pid in */
+    int     pidFileStrict;          /* TRUE if a preexisting pid file should cause an error. */
     TCHAR   *lockFilename;          /* Name of file to store wrapper lock in */
     TCHAR   *javaPidFilename;       /* Name of file to store jvm pid in */
     TCHAR   *javaIdFilename;        /* Name of file to store jvm id in */
@@ -398,6 +424,13 @@ struct WrapperConfig {
     int     signalUSR2Mode;         /* Controls what happens when the Wrapper receives a USR2 signal. */
     int     jvmStopped;             /* Flag which remembers the the stopped state of the JVM process. */
 #endif
+    
+    int pendingPingQueueOverflow;   /* Flag which is set to true if the PendingPingQueue overflows the limit of WRAPPER_MAX_PENDING_PINGS. */
+    int pendingPingQueueOverflowEmptied; /* Flag which is set when the queue size is reduced to 0 after having overflowed. */
+    int pendingPingCount;           /* Number of PendingPing events in the list. */
+    PPendingPing firstPendingPing;  /* Pointer to the first PendingPing in the list. */
+    PPendingPing firstUnwarnedPendingPing; /* Pointer to the first PendingPing in the list for which a slow warning has not been logged. */
+    PPendingPing lastPendingPing;   /* Pointer to the last PendingPing in the list. */
 
 #ifdef WIN32
     int     ctrlEventCTRLCTrapped;  /* CTRL_C_EVENT trapped. */
@@ -655,14 +688,20 @@ extern void wrapperJVMProcessExited(TICKS nowTicks, int exitCode);
 
 /**
  * Read and process any output from the child JVM Process.
- * Most output should be logged to the wrapper log file.
  *
- * This function will only be allowed to run for 250ms before returning.  This is to
- *  make sure that the main loop gets CPU.  If there is more data in the pipe, then
- *  the function returns TRUE, otherwise FALSE.  This is a hint to the mail loop not to
- *  sleep.
+ * When maxTimeMS is non-zero this function will only be allowed to run for that maximum
+ *  amount of time.  This is done to make sure the calling function is allowed CPU for
+ *  other activities.   When timing out for this reason when there is more data in the
+ *  pipe, this function will return TRUE to let the calling code know that it should
+ *  not to any unnecessary sleeps.  Otherwise FALSE will be returned.
+ *
+ * @param maxTimeMS The maximum number of milliseconds that this function will be allowed
+ *                  to run without returning.  In reality no new reads will happen after
+ *                  this time, but actual processing may take longer.
+ *
+ * @return TRUE if the calling code should call this function again as soon as possible.
  */
-extern int wrapperReadChildOutput();
+extern int wrapperReadChildOutput(int maxTimeMS);
 
 /**
  * Changes the current Wrapper state.
@@ -716,11 +755,6 @@ extern void wrapperMaintainSignals();
 extern TCHAR* findPathOf(const TCHAR *exe, const TCHAR* name);
 #endif
 
-/**
- * Gets the error code for the last operation that failed.
- */
-extern int wrapperGetLastError();
-
 /**
  * Execute initialization code to get the wrapper set up.
  */
@@ -963,8 +997,11 @@ extern TICKS wrapperAddToTicks(TICKS start, int seconds);
  * Sets the working directory of the Wrapper to the specified directory.
  *  The directory can be relative or absolute.
  * If there are any problems then a non-zero value will be returned.
+ *
+ * @param dir Directory to change to.
+ * @param logErrors TRUE if errors should be logged.
  */
-extern int wrapperSetWorkingDir(const TCHAR* dir);
+extern int wrapperSetWorkingDir(const TCHAR* dir, int logErrors);
 
 /******************************************************************************
  * Protocol callback functions
@@ -972,12 +1009,27 @@ extern int wrapperSetWorkingDir(const TCHAR* dir);
 extern void wrapperLogSignaled(int logLevel, TCHAR *msg);
 extern void wrapperKeyRegistered(TCHAR *key);
 
+/**
+ * Called when a ping is first determined to be slower than the wrapper.ping.alert.threshold.
+ *  This will happen before it has actually been responded to.
+ */
+extern void wrapperPingSlow();
+
+/**
+ * Called when a ping is responded to, but was slower than the wrapper.ping.alert.threshold.
+ *
+ * @param tickAge The number of seconds it took to respond.
+ */
+extern void wrapperPingRespondedSlow(int tickAge);
+
 /**
  * Called when a ping response is received.
  *
  * @param pingSendTicks Time in ticks when the ping was originally sent.
+ * @param queueWarnings TRUE if warnings about the queue should be logged, FALSE if the ping response did not contain a time.
  */
-extern void wrapperPingResponded(TICKS pingSendTicks);
+extern void wrapperPingResponded(TICKS pingSendTicks, int queueWarnings);
+
 extern void wrapperPingTimeoutResponded();
 extern void wrapperStopRequested(int exitCode);
 extern void wrapperRestartRequested();
@@ -1003,6 +1055,6 @@ struct LoadParameterFileCallbackParam {
 };
 
 #ifdef CUNIT
-extern void testJavaAdditionalParamSuite(void);
+extern void tsJAP_testJavaAdditionalParamSuite(void);
 #endif /* CUNIT */
 #endif
